package ca.uvic.cs.sugilayout.algorithms;


import org.eclipse.mylar.zest.layouts.algorithms.AbstractLayoutAlgorithm;
import org.eclipse.mylar.zest.layouts.dataStructures.DisplayIndependentRectangle;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalNode;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalRelationship;

import uk.ac.ebi.interpro.graphdraw.HierarchicalLayout;

/**
 * Main driver for sugiyama hierarchical layout with bent edges
 * @author Chris Bennett
 */
public class SugiLayoutAlgorithm extends AbstractLayoutAlgorithm {

	private DisplayIndependentRectangle bounds = null;
	private HierarchicalLayout layout;

	/**
	 * Default constructor
	 * 
	 * @param styles
	 */
	public SugiLayoutAlgorithm(int styles) {
		super(styles);
		resizeEntitiesAfterLayout = false;
	}

	protected int getCurrentLayoutStep() {
		return 0;
	}

	protected int getTotalNumberOfLayoutSteps() {
		return 0;
	}

	/**
	 * Copied from Grid layout
	 */
	protected boolean isValidConfiguration(boolean asynchronous,
			boolean continuous) {
		if (asynchronous && continuous)
			return false;
		else if (asynchronous && !continuous)
			return true;
		else if (!asynchronous && continuous)
			return false;
		else if (!asynchronous && !continuous)
			return true;
		return false;
	}

	protected void preLayoutAlgorithm(InternalNode[] entitiesToLayout,
			InternalRelationship[] relationshipsToConsider, double x, double y,
			double width, double height) {
		setLayoutArea(x, y, width, height);
		layout = new HierarchicalLayout(entitiesToLayout, relationshipsToConsider);
	}

	/**
	 * Set the layout bounds
	 */
	public void setLayoutArea(double x, double y, double width, double height) {
		this.bounds = new DisplayIndependentRectangle(x, y, width, height);
	}

	/**
	 * Apply the layout, using the Zanthan classes
	 */
	protected void applyLayoutInternal(InternalNode[] entitiesToLayout,
			InternalRelationship[] relationshipsToConsider, double boundsX,
			double boundsY, double boundsWidth, double boundsHeight) {
		layout.layout();
	}

	/**
	 * Adjust size to physical coords and save bendpoints and locations
	 * following layout.
	 */
	protected void postLayoutAlgorithm(InternalNode[] entitiesToLayout,
			InternalRelationship[] relationshipsToConsider) {
		defaultFitWithinBounds(entitiesToLayout, relationshipsToConsider,
				bounds);
		updateBendPoints(relationshipsToConsider);
		updateLayoutLocations(entitiesToLayout);
	}
}