/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.ShrimpApplication;


/**
 * The purpose of this class is to provide global access to the "one and only" shrimp application.
 * This class is neccessary because Java does not allow static methods to be defined in interfaces
 * such as our ShrimpApplication interface. We allow this global access to the application because 
 * there will only ever be one application in a single session. 
 * 
 * There are other ways to implement a singleton in Java, but we chose this way because it 
 * is easy to understand. 
 * 
 * One alternative would be to define a static getApplication method in AbstractShrimpApplication 
 * be we we do not want to cast every from ShrimpApplication to AbstractShrimpApplication. 
 * 
 * @author Rob Lintern
 */
public class ApplicationAccessor {
	
	/** The one and only shrimp application */
	private static ShrimpApplication app;

	/** 
	 * Sets the one and only shrimp application.
	 */
	public static void setApplication(ShrimpApplication application) {
		app = application;
	}
	
	/**
	 * Returns the one and only shrimp application.
	 * NOTE: <code>setApplication</code> must be called before calling this method.
	 */
	public static ShrimpApplication getApplication() {
		if (app == null) {
			(new Exception ("Must call setApplication once before calling getApplication!")).printStackTrace();
		}
		return app;
	}
	
	/**
	 * Checks if the application is set (not null).
	 * @return boolean
	 */
	public static boolean isApplicationSet() {
		return (app != null);
	}
	
}
